package cn.ibizlab.api.rest;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.math.BigInteger;
import java.util.HashMap;
import lombok.extern.slf4j.Slf4j;
import com.alibaba.fastjson.JSONObject;
import javax.servlet.ServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.http.HttpStatus;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.StringUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.access.prepost.PostAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import cn.ibizlab.api.dto.*;
import cn.ibizlab.api.mapping.*;
import cn.ibizlab.core.task.domain.JOBInfo;
import cn.ibizlab.core.task.service.IJOBInfoService;
import cn.ibizlab.core.task.filter.JOBInfoSearchContext;

@Slf4j
@Api(tags = {"任务信息" })
@RestController("api-jobinfo")
@RequestMapping("")
public class JOBInfoResource {

    @Autowired
    public IJOBInfoService jobinfoService;

    @Autowired
    @Lazy
    public JOBInfoMapping jobinfoMapping;

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBInfo-Update-all')")
    @ApiOperation(value = "更新任务信息", tags = {"任务信息" },  notes = "更新任务信息")
	@RequestMapping(method = RequestMethod.PUT, value = "/jobinfos/{jobinfo_id}")
    @Transactional
    public ResponseEntity<JOBInfoDTO> update(@PathVariable("jobinfo_id") BigInteger jobinfo_id, @RequestBody JOBInfoDTO jobinfodto) {
		JOBInfo domain  = jobinfoMapping.toDomain(jobinfodto);
        domain .setId(jobinfo_id);
		jobinfoService.update(domain );
		JOBInfoDTO dto = jobinfoMapping.toDto(domain );
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBInfo-Update-all')")
    @ApiOperation(value = "批量更新任务信息", tags = {"任务信息" },  notes = "批量更新任务信息")
	@RequestMapping(method = RequestMethod.PUT, value = "/jobinfos/batch")
    public ResponseEntity<Boolean> updateBatch(@RequestBody List<JOBInfoDTO> jobinfodtos) {
        jobinfoService.updateBatch(jobinfoMapping.toDomain(jobinfodtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @ApiOperation(value = "检查任务信息", tags = {"任务信息" },  notes = "检查任务信息")
	@RequestMapping(method = RequestMethod.POST, value = "/jobinfos/checkkey")
    public ResponseEntity<Boolean> checkKey(@RequestBody JOBInfoDTO jobinfodto) {
        return  ResponseEntity.status(HttpStatus.OK).body(jobinfoService.checkKey(jobinfoMapping.toDomain(jobinfodto)));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBInfo-Save-all')")
    @ApiOperation(value = "保存任务信息", tags = {"任务信息" },  notes = "保存任务信息")
	@RequestMapping(method = RequestMethod.POST, value = "/jobinfos/save")
    public ResponseEntity<Boolean> save(@RequestBody JOBInfoDTO jobinfodto) {
        return ResponseEntity.status(HttpStatus.OK).body(jobinfoService.save(jobinfoMapping.toDomain(jobinfodto)));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBInfo-Save-all')")
    @ApiOperation(value = "批量保存任务信息", tags = {"任务信息" },  notes = "批量保存任务信息")
	@RequestMapping(method = RequestMethod.POST, value = "/jobinfos/savebatch")
    public ResponseEntity<Boolean> saveBatch(@RequestBody List<JOBInfoDTO> jobinfodtos) {
        jobinfoService.saveBatch(jobinfoMapping.toDomain(jobinfodtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @ApiOperation(value = "获取任务信息草稿", tags = {"任务信息" },  notes = "获取任务信息草稿")
	@RequestMapping(method = RequestMethod.GET, value = "/jobinfos/getdraft")
    public ResponseEntity<JOBInfoDTO> getDraft() {
        return ResponseEntity.status(HttpStatus.OK).body(jobinfoMapping.toDto(jobinfoService.getDraft(new JOBInfo())));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBInfo-Remove-all')")
    @ApiOperation(value = "删除任务信息", tags = {"任务信息" },  notes = "删除任务信息")
	@RequestMapping(method = RequestMethod.DELETE, value = "/jobinfos/{jobinfo_id}")
    @Transactional
    public ResponseEntity<Boolean> remove(@PathVariable("jobinfo_id") BigInteger jobinfo_id) {
         return ResponseEntity.status(HttpStatus.OK).body(jobinfoService.remove(jobinfo_id));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBInfo-Remove-all')")
    @ApiOperation(value = "批量删除任务信息", tags = {"任务信息" },  notes = "批量删除任务信息")
	@RequestMapping(method = RequestMethod.DELETE, value = "/jobinfos/batch")
    public ResponseEntity<Boolean> removeBatch(@RequestBody List<BigInteger> ids) {
        jobinfoService.removeBatch(ids);
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBInfo-Get-all')")
    @ApiOperation(value = "获取任务信息", tags = {"任务信息" },  notes = "获取任务信息")
	@RequestMapping(method = RequestMethod.GET, value = "/jobinfos/{jobinfo_id}")
    public ResponseEntity<JOBInfoDTO> get(@PathVariable("jobinfo_id") BigInteger jobinfo_id) {
        JOBInfo domain = jobinfoService.get(jobinfo_id);
        JOBInfoDTO dto = jobinfoMapping.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBInfo-Create-all')")
    @ApiOperation(value = "新建任务信息", tags = {"任务信息" },  notes = "新建任务信息")
	@RequestMapping(method = RequestMethod.POST, value = "/jobinfos")
    @Transactional
    public ResponseEntity<JOBInfoDTO> create(@RequestBody JOBInfoDTO jobinfodto) {
        JOBInfo domain = jobinfoMapping.toDomain(jobinfodto);
		jobinfoService.create(domain);
        JOBInfoDTO dto = jobinfoMapping.toDto(domain);
		return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBInfo-Create-all')")
    @ApiOperation(value = "批量新建任务信息", tags = {"任务信息" },  notes = "批量新建任务信息")
	@RequestMapping(method = RequestMethod.POST, value = "/jobinfos/batch")
    public ResponseEntity<Boolean> createBatch(@RequestBody List<JOBInfoDTO> jobinfodtos) {
        jobinfoService.createBatch(jobinfoMapping.toDomain(jobinfodtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBInfo-Default-all')")
	@ApiOperation(value = "获取DEFAULT", tags = {"任务信息" } ,notes = "获取DEFAULT")
    @RequestMapping(method= RequestMethod.GET , value="/jobinfos/fetchdefault")
	public ResponseEntity<List<JOBInfoDTO>> fetchDefault(JOBInfoSearchContext context) {
        Page<JOBInfo> domains = jobinfoService.searchDefault(context) ;
        List<JOBInfoDTO> list = jobinfoMapping.toDto(domains.getContent());
        return ResponseEntity.status(HttpStatus.OK)
                .header("x-page", String.valueOf(context.getPageable().getPageNumber()))
                .header("x-per-page", String.valueOf(context.getPageable().getPageSize()))
                .header("x-total", String.valueOf(domains.getTotalElements()))
                .body(list);
	}

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibztask-JOBInfo-Default-all')")
	@ApiOperation(value = "查询DEFAULT", tags = {"任务信息" } ,notes = "查询DEFAULT")
    @RequestMapping(method= RequestMethod.POST , value="/jobinfos/searchdefault")
	public ResponseEntity<Page<JOBInfoDTO>> searchDefault(@RequestBody JOBInfoSearchContext context) {
        Page<JOBInfo> domains = jobinfoService.searchDefault(context) ;
	    return ResponseEntity.status(HttpStatus.OK)
                .body(new PageImpl(jobinfoMapping.toDto(domains.getContent()), context.getPageable(), domains.getTotalElements()));
	}
}

